;Just as a simple example: My own gauss approx. filter for
;images
;Don't forget to compile -> enormeous speed-up!
(defun smooth_pixel (arr i j)
       (setf (aref arr i j) 
       	     (/ (+ (* 4 (aref arr i       j      ))
       	     	   (* 2 (aref arr (+ i 1) j      ))
	     	   (* 2 (aref arr (- i 1) j      ))
	     	   (* 2 (aref arr i       (+ j 1)))
	     	   (* 2 (aref arr i       (- j 1)))
       	     	   (* 1 (aref arr (+ i 1) (+ j 1)))
	     	   (* 1 (aref arr (+ i 1) (- j 1)))
	     	   (* 1 (aref arr (- i 1) (+ j 1)))
	     	   (* 1 (aref arr (- i 1) (- j 1))))
               16)))
(compile 'smooth_pixel)
(defun mysmooth (arr)
      (do* ((j 1 (+ j 1)))
      	  ((= j (- (array-dimension arr 1) 1)))
	      (do* ((i 1 (+ i 1)))
      	      	  ((= i (- (array-dimension arr 0) 1)))
		      (smooth_pixel arr i j))))
(compile 'mysmooth)

;Always the first step: load the VIGRA CL Lib
(load "vigracl.cl")

(print "loading blox-image")
(time (setq img (vigracl.impex:loadimage  "blox.gif")))

(print "performance test gaussian smoothing")
(print "vigra-method:")
(time (vigracl.filters:gsmooth img 0.3))
(print "my-own-method:")
;(time (mysmooth img))


(print "performing watershed transform on resized gradient image")
(time (setq img2  (vigracl.segmentation:labelimage
	    (vigracl.segmentation:watersheds
	     (vigracl.filters:ggradient 
	      (vigracl.imgproc:resizeimage img 800 800 4)
	      3.0)))))

(print "performing fft on image")
(time (setq img3 (vigracl.imgproc:fouriertransform img 800 800)))

(print "testing rotation and reflection functions on image")
(time (setq img4 (vigracl.imgproc:reflectimage img 3)))
(time (setq img5 (vigracl.imgproc:rotateimage img 15.0 3)))

(print "performing distance transform on canny edges of image")
(time (setq img6 (vigracl.filters:distancetransform
		  (vigracl.segmentation:cannyedgeimage img 1.8 0.1 100.0)
		  0.0 2)))

(print "testing difference of exponential edge detection on image")
(time (setq img7 (vigracl.segmentation:differenceofexponentialedgeimage img 0.8 0.1 100.0)))

(print "testing nearly all filters")
(time (setq img8 (vigracl.filters:gsmooth img 3.0)))
(time (setq img9 (vigracl.filters:laplacianofgaussian img 3.0)))
;(time (setq img10 (vigracl.filters:gsharpening img 0.5 3.0)))
;(time (setq img11 (vigracl.filters:sharpening img 3.0)))
;(time (setq img12 (vigracl.filters:nonlineardiffusion img 0.1 2.0)))

(print "saving resulting images")
(vigracl.impex:saveimage img2  "blox-relabeled-watersheds-on-resized-gradient-image.png")

(vigracl.impex:saveimage (car img3) "blox-fft-real.png")
(vigracl.impex:saveimage (cdr img3) "blox-fft-imag.png")

(vigracl.impex:saveimage img4  "blox-reflected-both.png")
(vigracl.impex:saveimage img5  "blox-rotated-15deg.png")
(vigracl.impex:saveimage img6  "blox-disttransform-on-canny.png")
(vigracl.impex:saveimage img7  "blox-diff_of_exp.png")
(vigracl.impex:saveimage img8  "blox-gsmooth-3.0.png")
(vigracl.impex:saveimage img9  "blox-log-3.0.png")
;(vigracl.impex:saveimage img10 "blox-gsharpening-0.5-3.0.png")
;(vigracl.impex:saveimage img11 "blox-sharpening-3.0.png")
;(vigracl.impex:saveimage img12 "blox-nonlineardiffusion-0.1-2.0.png")